/// Header files.
#include <dshow.h>
#include <windows.h>

/// Macros.
#define MAX_VOL		0
#define MIN_VOL		-10000

/// Operation to perform.
typedef enum tOperation
{
	PLAY = 1,
	PAUSE,
	STOP,
	VOLUME_UP,
	VOLUME_DOWN,
	EXIT
}operation;

int wmain(void)
{
	/// Declarations.
	IGraphBuilder *pGraph = NULL;
	IMediaControl *pMediaControl = NULL;
	IBasicAudio *pAudio = NULL;
	IMediaEvent *pEvent = NULL;
	IMediaSeeking *pSeek = NULL;
	HRESULT hResult = S_OK;
	OAFilterState filterState = 0;
	LONG volume = 0;
	DWORD select = 0;
	LONGLONG startPosition = 0;
	LONGLONG stopPosition = 0;
	LONGLONG currentPosition = 0;

	/// COM initialize.
	CoInitialize(NULL);
	/// Create the filter graph manager and query for interfaces. 
	hResult = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void**)&pGraph);
	if (hResult != S_OK)
	{
		printf("\nFilter graph manager creation failed with error code = %#x", hResult);
		fflush(stdin);
		printf("\nPress enter to exit.");
		getchar();
		/// Release COM pointer.
		CoUninitialize();
		return 0;
	}
	/// Link the controls and events to the graph.
	pGraph->QueryInterface(IID_IMediaControl, (VOID **)&pMediaControl); 
	pGraph->QueryInterface(IID_IMediaEvent, (VOID **)&pEvent);
	pGraph->QueryInterface(IID_IBasicAudio,(VOID **)&pAudio);
	pGraph->QueryInterface(IID_IMediaSeeking,(VOID **)&pSeek);
	/// Build the graph. 
	hResult = pGraph->RenderFile(L"\\Program Files\\vcpp_audio_playback_ds\\Sleep_Away.mp3", NULL); 
	if (hResult != S_OK)
	{
		printf("\nRendering failed with error = %#x", hResult);
		fflush(stdin);
		printf("\nPress enter to exit.");
		getchar();
		/// Clean up. 
		pMediaControl->Release(); 
		pEvent->Release(); 
		pAudio->Release();
		pSeek->Release();
		pGraph->Release();
		/// Release COM pointer.
		CoUninitialize();
		return 0;
	}

	/// Get stop duration of media stream.
	pSeek->GetStopPosition(&stopPosition);

	while (select != EXIT)
	{
	printf("\n\n********************** Menu **********************");
	printf("\n1. Play \n2. Pause \n3. Stop \n4. Volume Up \n5. Volume Down \n6. Exit\t ");
	fflush(stdin);
	scanf_s("%u", &select);
	switch(select)
	{
	case PLAY:
		{
			/// Get current position of the media stream.
			pSeek->GetCurrentPosition(&currentPosition);
			/// If media stream finishes but graph is running.
			if (currentPosition >= stopPosition)
			{
				/// Stop rendering.
				pMediaControl->Stop();
				/// Set current position to start of the stream.
				pSeek->SetPositions(&startPosition, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
			}
			/// Get state of the graph i.e. Stopped, Paused, or Running.
			pMediaControl->GetState(INFINITE, &filterState);
			if (filterState == State_Running)
			{
				printf("\nAlready in play mode.");
				break;
			}
			/// Start rendering.
			hResult = pMediaControl->Run();
			printf("\nPlaying track...");
			Sleep(200);
			break;
		}
	case PAUSE:
		{
			/// Get state of the graph i.e. Stopped, Paused, or Running.
			pMediaControl->GetState(INFINITE, &filterState);
			if (filterState != State_Running)
			{
				printf("\nFirst play the track.");
				break;
			}
			/// Pause rendering.
			pMediaControl->Pause(); 
			printf("\nPaused.");
			Sleep(200);
			break;
		}
	case STOP:
		{
			/// Get state of the graph i.e. Stopped, Paused, or Running.
			pMediaControl->GetState(INFINITE, &filterState);
			if (filterState == State_Stopped)
			{
				printf("\nFirst play the track.");
				break;
			}
			/// Stop rendering.
			pMediaControl->Stop();
			/// Set current position to start of the stream.
			pSeek->SetPositions(&startPosition, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
			printf("\nStopped.");
			Sleep(200);
			break;
		}
	case VOLUME_UP:
		{
			/// Get current volume.
			pAudio->get_Volume(&volume);
			if (volume == MAX_VOL)
			{
				printf("\nMaximum volume.");
				break;
			}
			volume += 250;
			/// Set the volume.
			pAudio->put_Volume(volume);
			printf("\nVolume increased.");
			Sleep(200);
			break;
		}
	case VOLUME_DOWN:
		{
			/// Get current volume.
			pAudio->get_Volume(&volume);
			if (volume == MIN_VOL)
			{
				printf("\nMinimum volume.");
				break;
			}
			volume -= 250;
			/// Set the volume.
			pAudio->put_Volume(volume);
			printf("\nVolume decreased.");
			Sleep(200);
			break;
		}
	case EXIT:
		{
			break;
		}
	default:
		{
			printf("\nInvalid selection. Try again.");
		}
	}
	}
	/// Clean up. 
	pMediaControl->Release(); 
	pEvent->Release(); 
	pAudio->Release();
	pSeek->Release();
	pGraph->Release();
	/// Release COM pointer.
	CoUninitialize();
	return 0;
}